/*
 *  SSObject.cpp
 *  SlingShot
 *
 *  Created by Keun Sup Lee on 2/6/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#import "SSObject.h"
#import <math.h>



extern Vector3D g_gravity;
extern Float32 g_wallResist;
extern Float32 g_airResist;


//---------------------------------------------------------------------
// name: rotatePointXY()
// description: Rotate a point clockwise on XY plane by theta degree
//---------------------------------------------------------------------

Vector3D rotatePointXY(GLfloat theta, Vector3D point)
{
	Vector3D outPoint;
	GLfloat radian = theta*M_PI/180.f;
	
	outPoint.x = point.x * cos(radian) + point.y * sin(radian);
	outPoint.y = point.x * -sin(radian) + point.y * cos(radian);
	outPoint.z = 0;
	
	return outPoint;
}


//---------------------------------------------------------------------
// name: drawLineXY()
// description: Calculate values of vertices to draw a line
//---------------------------------------------------------------------

void drawLineXY(int samples, SAMPLE *vertices, Vector3D pStart, Vector3D pEnd, GLfloat xStep)
{
	SAMPLE x = pStart.x;
	SAMPLE y = pStart.y;
	
	if( samples == 1 ) {
		vertices[0] = x;
		vertices[1] = y;	
	} else {
		SAMPLE yinc = (pEnd.y - pStart.y)/(samples-1);
	
		for( int i=0; i < samples; i++ ){
			vertices[2*i] = x; x+= xStep;	//xinc;
			vertices[2*i+1] = y; y += yinc;
		}
	}
}


//---------------------------------------------------------------------
// name: drawEllipseXY()
// description: Draw an ellipse
//---------------------------------------------------------------------

void drawEllipseXY(int segments, CGFloat width, CGFloat height, Vector3D center, bool filled)
{
	glPushMatrix();
	
	glTranslatef(center.x, center.y, 0);
	
	GLfloat vertices[segments*2];
	int count=0;
	
	for(int i=0; i < segments; i++){
		vertices[count++] = cos(TWO_PI*i/segments)*width;
		vertices[count++] = sin(TWO_PI*i/segments)*height;
	}
	
	glVertexPointer (2, GL_FLOAT , 0, vertices);
	glDrawArrays ((filled) ? GL_TRIANGLE_FAN : GL_LINE_LOOP, 0, segments);
	
	glPopMatrix();
}


//---------------------------------------------------------------------
// name: drawCircleXY()
// description: Draw an circle
//---------------------------------------------------------------------

void drawCircleXY(int circleSegments, CGFloat circleSize, Vector3D center, bool filled) 
{
	if( circleSize <= 0 )
		return;
	
	drawEllipseXY(circleSegments, circleSize, circleSize, center, filled);
}




//---------------------------------------------------------------------
// name: SSProjectile()
// description: 
//---------------------------------------------------------------------

SSProjectile::SSProjectile()
{
	pos.set(0, 0, 0);
	vel.setAll(0);
	bullet = false;
	
	justRemove = false;
	collisionOn = false;
	collisionCount = 0;
	explosionOn = false;
	explosionFrame = 0;;
	explosionSlow = 0;
}


//---------------------------------------------------------------------
// name: SSProjectile()
// description: 
//---------------------------------------------------------------------

SSProjectile::SSProjectile(Vector3D position)
{
	position.x = MIN(MAX(position.x, 0.f), Y_RESOL);
	position.y = MIN(MAX(position.y, 0.f), X_RESOL);
	
	pos.set(XAXIS_MAPPING(position.y), YAXIS_MAPPING(position.x), 0);
	vel.setAll(0);	
	bullet = false;
	
	justRemove = false;
	collisionOn = false;
	collisionCount = 0;
	explosionOn = false;
	explosionFrame = 0;;
	explosionSlow = 0;
}


//---------------------------------------------------------------------
// name: setPoint()
// description: 
//---------------------------------------------------------------------

void SSProjectile::setPoint(CGPoint point)
{
	point.x = MIN(MAX(point.x, 0.f), Y_RESOL);
	point.y = MIN(MAX(point.y, 0.f), X_RESOL);
	
	GLfloat x = XAXIS_MAPPING(point.y);
	GLfloat y = YAXIS_MAPPING(point.x);
	
	pos.set(MIN(MAX(x, -1.5f+PROJECTILE_RADIUS), 1.5f-PROJECTILE_RADIUS),
			MIN(MAX(y, -1.f+PROJECTILE_RADIUS), 1.f-PROJECTILE_RADIUS), 0);
}


void SSProjectile::setPoint(Vector3D point)
{
	pos.set(MIN(MAX(point.x, -1.5f+PROJECTILE_RADIUS), 1.5f-PROJECTILE_RADIUS),
			MIN(MAX(point.y, -1.f+PROJECTILE_RADIUS), 1.f-PROJECTILE_RADIUS), 0);
}


//---------------------------------------------------------------------
// name: setInitVelocity()
// description: 
//---------------------------------------------------------------------

void SSProjectile::setInitVelocity(Vector3D velocity)
{
	if( bullet ) {
		velocity.normalize();
		velocity *= .15f;
		vel = velocity;
	} else {
		vel = velocity;
	}
}



//---------------------------------------------------------------------
// name: setVelocity()
// description: 
//---------------------------------------------------------------------

void SSProjectile::setVelocity(CGPoint velocity)
{
	velocity.x = MIN(MAX(velocity.x, 0.f), Y_RESOL);
	velocity.y = MIN(MAX(velocity.y, 0.f), X_RESOL);
	
	vel.set(XAXIS_MAPPING(velocity.y), YAXIS_MAPPING(velocity.x), 0);
}



//---------------------------------------------------------------------
// name: draw()
// description: 
//---------------------------------------------------------------------

int SSProjectile::draw(int themeType, GLuint *texture, struct AudioData *audioData)
{
	int returnValue=NOP;
	Vector3D frictionFactor(.9f, .9f, 1.f);
	
	static const GLfloat projectileVertices[] = {
        -0.15f,  -0.15f,
		0.15f,  -0.15f,
        -0.15f,   0.15f,
		0.15f,   0.15f,
    };
	
	static const GLfloat explosionVertices[] = {
        -0.3f,  -0.3f,
		0.3f,  -0.3f,
        -0.3f,   0.3f,
		0.3f,   0.3f,
    };	
	
	static const GLfloat texCoords[] = {
		0, 1,
		1, 1,
		0, 0,
		1, 0,
	};
	
	if( justRemove ) {
		returnValue = DELETE_PROJECTILE;
		
		return returnValue;
	}
	
	// explosion effect
	if( explosionOn ) {

		glEnable( GL_TEXTURE_2D );
		glBindTexture( GL_TEXTURE_2D, texture[TEX_EXPLOSION+explosionFrame] );
		
		glPushMatrix();
		
		glTranslatef(pos.x, pos.y, 0);
		
		glColor4f(1.f, 1.f, 1.f, 1.f);
		
		glVertexPointer(2, GL_FLOAT, 0, explosionVertices);
		glEnableClientState(GL_VERTEX_ARRAY);
		
		glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
		
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		
		glPopMatrix();
		
		glDisable( GL_TEXTURE_2D );
		
		if( ++explosionSlow % 5 == 0 )
			explosionFrame++;
		
		if( explosionFrame >= 16 ) {
			explosionOn = false;
			
			returnValue = DELETE_PROJECTILE;
		}
		
		return returnValue;
		
	} else {
		if( themeType == 0 ) {		// no texture

			if( collisionOn ) {
				
				glPushMatrix();
				
				// blinking effect when a collision happens
				if( (++collisionCount) & 0x1 ) {
					glColor4f(1.f, 1.f, 1.f, 1.f);
					drawCircleXY(100, .1f, pos, true);

					glColor4f(.25f, .70f, .75f, .5f);
					drawCircleXY(100, .065f, pos, true);
					
					glColor4f(.75f, .75f, .55f, 1.f);
					drawCircleXY(100, .065f, pos, false);
				} else {
				
					glColor4f(.25f, .70f, .75f, .5f);
					drawCircleXY(100, .1f, pos, true);

					glColor4f(.75f, .75f, .55f, 1.f);
					drawCircleXY(100, .065f, pos, false);
				}
				
				glPopMatrix();
				
				if( collisionCount >= 10 ) {
					collisionCount = 0;
					collisionOn = false;
				}
				
			} else {
				glPushMatrix();
			
				glColor4f(.25f, .70f, .75f, .5f);
				drawCircleXY(100, .1f, pos, true);
				glColor4f(.75f, .75f, .55f, 1.f);
				drawCircleXY(100, .065f, pos, false);
			
				glPopMatrix();
			}
		} else {		// star wars theme
			
			// no collision effect for star wars theme
			if( collisionOn )
				collisionOn = false;
			
			if( bullet ) {
				SAMPLE bulletVertices[4];
				Vector3D dif=vel;
				dif.normalize();
				dif *= .2f;
				
				bulletVertices[0]=0;
				bulletVertices[1]=0;
				bulletVertices[2]=dif.x;
				bulletVertices[3]=dif.y;
				
				glPushMatrix();

				glTranslatef(pos.x, pos.y, 0);

				glVertexPointer(2, GL_FLOAT, 0, bulletVertices);
				
				glColor4f(0.f, .5f, 0.f, .5f);
				glLineWidth(10);
				glDrawArrays(GL_LINE_STRIP, 0, 2);
					
				glColor4f(.25f, 1.f, .25f, .5f);
				glLineWidth(7);
				glDrawArrays(GL_LINE_STRIP, 0, 2);
					
				glColor4f(1.f, 1.f, 1.f, 1.f);
				glLineWidth(5);
				glDrawArrays(GL_LINE_STRIP, 0, 2);
					
				glPopMatrix();
				
			} else {
				glEnable( GL_TEXTURE_2D );
				glBindTexture( GL_TEXTURE_2D, texture[TEX_PROJECTILE] );
		
				glPushMatrix();
			
				glTranslatef(pos.x, pos.y, 0);
			
				glColor4f(1.f, 1.f, 1.f, 1.f);
			
				glVertexPointer(2, GL_FLOAT, 0, projectileVertices);
				glEnableClientState(GL_VERTEX_ARRAY);
			
				glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			
				glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			
				glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			
				glPopMatrix();
			
				glDisable( GL_TEXTURE_2D );
			}
		}
	}
	
	pos += vel;
	
	if( !bullet ) {
		vel += g_gravity*0.02f;	// acceleration of gravity
		vel *= g_airResist;
	}
	
	// check wall reflection
	if( bullet ) {
		
		if( fabs(pos.x) > 1.5f ) {
			
			if( pos.x < 0.f )
				pos.x = -1.5f;
			else
				pos.x = 1.5f;
			
			vel.x *= -1.f;
			
			setCollision();
			
			audioData->cClip->startPlay();
		}
		
		if( fabs(pos.y) > 1.f ) {
			
			if( pos.y < 0.f )
				pos.y = -1.f;
			else
				pos.y = 1.f;
			
			vel.y *= -1.f;
			
			setCollision();
			
			audioData->cClip->startPlay();
		}
		
	} else {

		if( fabs(pos.x) > (1.5f-PROJECTILE_RADIUS) ) {

			if( pos.x < 0.f )
				pos.x = -(1.5f-PROJECTILE_RADIUS);
			else
				pos.x = (1.5f-PROJECTILE_RADIUS);

			vel.x *= -1.f;
		
			vel.x *= g_wallResist;
			vel.y *= frictionFactor.y;
		
			setCollision();
			
			if( vel.x > 0.03f )
				audioData->cClip->startPlay();
		}
	
		if( fabs(pos.y) > (1.f-PROJECTILE_RADIUS) ) {

			if( pos.y < 0.f )
				pos.y = -(1.f-PROJECTILE_RADIUS);
			else
				pos.y = (1.f-PROJECTILE_RADIUS);
		
			vel.y *= -1.f;
		
			vel.y *= g_wallResist;	
			vel.x *= frictionFactor.x;
		
			setCollision();
			
			if( vel.y > 0.03f )
				audioData->cClip->startPlay();
		}
	}
	
	// check blackhole
	if( pos.x > 1.25f && fabs(pos.y) > 0.75f ) {
		audioData->eClip->startPlay();

		explosionOn = true;		
	}
	
	return returnValue;
}



//---------------------------------------------------------------------
// name: SSRubberBand()
// description: 
//---------------------------------------------------------------------

SSRubberBand::SSRubberBand()
{
	theme = 0;
	stringMagnitude = 0;
	pulling = false;
	plucking = false;
	drawString = false;
	
	memset(vertices, 0, sizeof(SAMPLE)*2*NUM_STRING_SAMPLE);
	memset(delayBuf0, 0, sizeof(SAMPLE)*NUM_STRING_SAMPLE);
	memset(delayBuf1, 0, sizeof(SAMPLE)*NUM_STRING_SAMPLE);
	
}


//---------------------------------------------------------------------
// name: setPoint()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::setPoint(CGPoint point0, CGPoint point1)
{
	point0.x = MIN(MAX(point0.x, 0.f), Y_RESOL);
	point1.x = MIN(MAX(point1.x, 0.f), Y_RESOL);
	point0.y = MIN(MAX(point0.y, 0.f), X_RESOL);
	point1.y = MIN(MAX(point1.y, 0.f), X_RESOL);
	
	GLfloat fromX = XAXIS_MAPPING(point0.y);
	GLfloat fromY = YAXIS_MAPPING(point0.x);
	GLfloat toX = XAXIS_MAPPING(point1.y);
	GLfloat toY = YAXIS_MAPPING(point1.x);
	
	startPoint.set(fromX, fromY, 0);
	endPoint.set(toX, toY, 0);
	
	Vector3D diff = endPoint - startPoint;
	stringLength = diff.magnitude();

	// calculate the slope of the rubber band
	theta = ::asin(diff.y/stringLength)*180./M_PI;
	if( diff.x < 0 )
		theta = 180. - theta;
	
	drawString = true;
	
	audioData->pluck->setFrequency(SPEED_OF_SOUND/(2.f*stringLength*0.15f));
}


//---------------------------------------------------------------------
// name: pullingString()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::pullingString(CGPoint point)
{
	if( !pulling )
		return;
	
	point.x = MIN(MAX(point.x, 0.f), Y_RESOL);
	point.y = MIN(MAX(point.y, 0.f), X_RESOL);
	
	GLfloat pointX = XAXIS_MAPPING(point.y);
	GLfloat pointY = YAXIS_MAPPING(point.x);
	
	pullingPoint.set(pointX, pointY, 0);
	
	// calculate the direction of shot
	Vector3D v0 = startPoint - pullingPoint;
	Vector3D v1 = endPoint - pullingPoint;
	
	shotDirection = v0 + v1;
}



//---------------------------------------------------------------------
// name: checkPullingString()
// description: 
//---------------------------------------------------------------------

bool SSRubberBand::checkPullingString(CGPoint point)
{
	if( pulling )
		return true;
	
	point.x = MIN(MAX(point.x, 0.f), Y_RESOL);
	point.y = MIN(MAX(point.y, 0.f), X_RESOL);
	
	GLfloat pointX = XAXIS_MAPPING(point.y);
	GLfloat pointY = YAXIS_MAPPING(point.x);
	
	Vector3D diff = startPoint - endPoint;
	Float32 slope = diff.y/diff.x;
	Float32 y = slope * (pointX - startPoint.x) + startPoint.y;
	
	// if the distance b/w touched point and the string is less than .03,
	// then the string start to be pulled
	if( fabs(y - pointY) < 0.3f ) {
		plucking = false;
		pulling = true;
		
		return true;
	}
	
	return false;
}


//---------------------------------------------------------------------
// name: updateStringState()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::updateStringState()
{
	static float filtBuf=0.f;
	if( !plucking )
		return;
	
	int movingStep = (int)(NUM_STRING_SAMPLE/(stringLength*0.015f)/SRATE*SPEED_OF_SOUND + .5f);
	SAMPLE *tmpBuf = new SAMPLE[movingStep*2];
	
	for( int i=0; i < movingStep; i++ ){
		tmpBuf[movingStep-i-1] = -tension*delayBuf1[i];						// bridge
		tmpBuf[movingStep+i] = -tension*delayBuf0[NUM_STRING_SAMPLE-1-i];	// nut
	}
	
	// frequency-dependent filter: H(z) = (1 + z^-1)/2
	if( movingStep > 0 ) {
		for( int i=movingStep-1; i > 0; i-- ){
			tmpBuf[movingStep+i] += tmpBuf[movingStep+i-1];
			tmpBuf[movingStep+i] *= .5f;
		}
		tmpBuf[movingStep] += filtBuf;
		tmpBuf[movingStep] *= .5f;
		filtBuf = tmpBuf[2*movingStep-1];
	}
	
	// shift delay buffers
	for( int i=NUM_STRING_SAMPLE-1; i >= movingStep; i-- )
		delayBuf0[i] = delayBuf0[i-movingStep];		

	memcpy(&delayBuf1[0], &delayBuf1[movingStep], sizeof(SAMPLE)*(NUM_STRING_SAMPLE-movingStep));

	memcpy(delayBuf0, tmpBuf, sizeof(SAMPLE)*movingStep);
	memcpy(&delayBuf1[NUM_STRING_SAMPLE-movingStep], &tmpBuf[movingStep], sizeof(SAMPLE)*movingStep);
	
	// make vertices to draw the string
	for(int i=0; i < NUM_STRING_SAMPLE; i++ )
		vertices[2*i+1] = delayBuf0[i] + delayBuf1[i];
	
	delete[] tmpBuf;

}


//---------------------------------------------------------------------
// name: removeString()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::removeString()
{
	memset(vertices, 0, sizeof(SAMPLE)*2*NUM_STRING_SAMPLE);
	memset(delayBuf0, 0, sizeof(SAMPLE)*NUM_STRING_SAMPLE);
	memset(delayBuf1, 0, sizeof(SAMPLE)*NUM_STRING_SAMPLE);
	
	startPoint.setAll(0);
	endPoint.setAll(0);
	pullingPoint.setAll(0);
	shotDirection.setAll(0);
	
	pulling = false;
	drawString = false;
	plucking = false;
	
	audioData->pluck->pluck(0.);
}


//---------------------------------------------------------------------
// name: startPlucking()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::startPlucking()
{
	if( !pulling ) 
		return;
	
	plucking = true;
	pulling = false;
	shotDirection.setAll(0);

	audioData->pluck->noteOn(SPEED_OF_SOUND/(2.f*stringLength*0.15f), stringMagnitude);

}



//---------------------------------------------------------------------
// name: draw()
// description: 
//---------------------------------------------------------------------

void SSRubberBand::draw()
{
	
	SAMPLE x;
	SAMPLE xinc;
	
	static const GLfloat saberVertices[] = {
		-0.01f, -0.06f,
		0.29f, -0.06f,
		-0.01f, 0.04f,
		0.29f, 0.04f
    };
	
	static const GLfloat texCoordsSaber[] = {
		0, 0,
		141.f/256.f, 0,
		0, 28.f/256.f,
		141.f/256.f, 28.f/256.f,
	};
	
	
	if( !plucking ) {
		if( pulling ) {
		
		  // draw rubber band
		  Vector3D origin;
		  Vector3D diff;
		  Vector3D end = endPoint;
		  Vector3D pull = pullingPoint;		
		
		  pull -= startPoint;
		  pull = rotatePointXY(theta, pull);

		  end -= startPoint;
		  end = rotatePointXY(theta, end);
		
		  diff = end - origin;
		
		  GLfloat len = diff.magnitude();
		  GLfloat xinc = len/NUM_STRING_SAMPLE;
		  int nSamples = (int)(NUM_STRING_SAMPLE * pull.x / len);
		
		  pull.x = MIN(MAX(pull.x, 0.f), end.x);
		
		  // draw one line from start point to pulling point
		  drawLineXY(nSamples, vertices, origin, pull, xinc);
		
		  // draw another line from pulling point to end point
		  drawLineXY(NUM_STRING_SAMPLE-nSamples+1, &vertices[2*(nSamples-1)], pull, end, xinc);
		
		
		  // Fill delay buffers for a string modelling
		  for( int i=0; i < NUM_STRING_SAMPLE; i++ )
			  delayBuf0[i] = delayBuf1[i] = vertices[2*i+1]/2.f;
		
		  stringMagnitude = fabs(pull.y);
		
		} else {
			x = 0.f;
			xinc = stringLength/(NUM_STRING_SAMPLE-1);
	
			for(int i=0; i < NUM_STRING_SAMPLE; i++ ){
				vertices[2*i] = x; x += xinc;
				vertices[2*i+1] = 0.f;
			}
		
		}
	} else {
		x = 0.f;
		xinc = stringLength/(NUM_STRING_SAMPLE-1);
		
		for(int i=0; i < NUM_STRING_SAMPLE; i++ ){
			vertices[2*i] = x; x += xinc;
		}
		
	}
	
	
	if( drawString ) {
		
		glPushMatrix();
		
		glTranslatef(startPoint.x, startPoint.y, 0);
		glRotatef(theta, 0, 0, 1);
		glVertexPointer(2, GL_FLOAT, 0, vertices);
		
		if( theme == 1) {	// star wars theme
			glColor4f(0.f, 0.f, .5f, .5f);
			glLineWidth(12);
			glDrawArrays(GL_LINE_STRIP, 0, NUM_STRING_SAMPLE);
			
			glColor4f(.25f, .25f, 1.f, .5f);
			glLineWidth(9);
			glDrawArrays(GL_LINE_STRIP, 0, NUM_STRING_SAMPLE);
			
			glColor4f(1.f, 1.f, 1.f, 1.f);
			glLineWidth(5);
			glDrawArrays(GL_LINE_STRIP, 0, NUM_STRING_SAMPLE);
		} else {
			glColor4f(1.f, 1.f, .25f, 1.f);
			glLineWidth(3);
			glDrawArrays(GL_LINE_STRIP, 0, NUM_STRING_SAMPLE);
		}
		
		glPopMatrix();
		
		if( theme == 1 ) {	// star wars theme
			
			glEnable( GL_TEXTURE_2D );
			glBindTexture( GL_TEXTURE_2D, texture[0] );
			
			glPushMatrix();
			
			glTranslatef(startPoint.x, startPoint.y, 0);
			glRotatef(theta+180., 0, 0, 1);
			
			glColor4f(1.f, 1.f, 1.f, 1.f);
			
			glVertexPointer(2, GL_FLOAT, 0, saberVertices);
			glEnableClientState(GL_VERTEX_ARRAY);
			
			glTexCoordPointer(2, GL_FLOAT, 0, texCoordsSaber);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			
			glPopMatrix();
			
			glPushMatrix();
			
			glTranslatef(endPoint.x, endPoint.y, 0);
			glRotatef(theta, 0, 0, 1);
			
			glColor4f(1.f, 1.f, 1.f, 1.f);
			
			glVertexPointer(2, GL_FLOAT, 0, saberVertices);
			glEnableClientState(GL_VERTEX_ARRAY);
			
			glTexCoordPointer(2, GL_FLOAT, 0, texCoordsSaber);
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			
			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
			
			glPopMatrix();
			
			glDisable( GL_TEXTURE_2D );
			
		} else {
			
			glColor4f(.25f, .25f, 1.f, 1.f);
			drawCircleXY(50, .3f, startPoint, false);
			glColor4f(.25f, 1.f, .25f, 1.f);
			drawCircleXY(20, .1f, startPoint, true);
			
			glColor4f(.25f, .25f, 1.f, 1.f);
			drawCircleXY(50, .3f, endPoint, false);
			glColor4f(.25f, 1.f, .25f, 1.f);
			drawCircleXY(20, .1f, endPoint, true);
			
		}
	}
	
}
